/**
 *
 */
package edu.agh.sm2006.smschess.server.socket;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Arrays;

import org.easymock.EasyMock;
import org.junit.BeforeClass;
import org.junit.Test;

import edu.agh.sm2006.smschess.server.message.MessageParser;
import edu.agh.sm2006.smschess.server.message.client2server.ClientToServerMessage;

/**
 * @author Kuba Madej
 *
 */
public class SocketConnectorTest {
	private static SocketConnector connector;

	private static MessageParser parser;

	private static InputStream in;

	private static OutputStream out;

	@BeforeClass
	public static void setUp() throws Exception {
		connector = new SocketConnector(null);
		parser = EasyMock.createMock(MessageParser.class);
		connector.openConnection(parser);

		new Thread() {
			public void run() {
				connector.runReceiver();
			}
		}.start();

		Socket socket = new Socket("localhost", SocketConnector.DEFAULT_PORT);

		in = socket.getInputStream();
		out = socket.getOutputStream();

		Thread.sleep(1000);
	}

	/**
	 * Test method for
	 * {@link edu.agh.sm2006.smschess.server.socket.SocketConnector#sendMessage(java.lang.String, java.lang.String)}.
	 *
	 * @throws IOException
	 */
	@Test
	public void testSendMessage() throws Exception {
		connector.sendMessage("0", "alamakota");

		byte[] buf = new byte[100];

		int n = in.read(buf);

		assertEquals(10, n);

		byte[] received = new byte[10];

		System.arraycopy(buf, 0, received, 0, 10);

		assertTrue(Arrays.equals(received, "alamakota\n".getBytes()));
	}

	@Test
	public void testReceiveMessage() throws Exception {
		final String messagePayload = "payload";
		final String responseMessage = "response";

		ClientToServerMessage mockMessage = EasyMock
				.createMock(ClientToServerMessage.class);

		parser.parse(EasyMock.eq(messagePayload));
		EasyMock.expect(parser).andStubReturn(mockMessage);
		EasyMock.replay(parser);

		mockMessage.setSenderId(EasyMock.eq("0"));
		mockMessage.process();
		EasyMock.expect(mockMessage).andStubReturn(responseMessage);
		EasyMock.replay(mockMessage);

		out.write((messagePayload + "\n").getBytes());
		out.flush();

		Thread.sleep(1000);

		EasyMock.verify(parser, mockMessage);

		byte[] buf = new byte[100];

		int n = in.read(buf);

		assertEquals(9, n);

		byte[] response = new byte[9];

		System.arraycopy(buf, 0, response, 0, 9);

		assertTrue(Arrays.equals(response, (responseMessage + "\n").getBytes()));
	}
}
