/**
 *
 */
package edu.agh.sm2006.smschess.server;

import java.util.Properties;

import org.apache.log4j.Logger;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;

import edu.agh.sm2006.smschess.server.config.Configuration;
import edu.agh.sm2006.smschess.server.message.MessageParser;
import edu.agh.sm2006.smschess.server.message.MessageParserImpl;
import edu.agh.sm2006.smschess.server.session.SessionManager;
import edu.agh.sm2006.smschess.server.session.XBoardSession;
import edu.agh.sm2006.smschess.server.session.xboard.XBoardEngineProcessManager;

/**
 * @author Kuba Madej
 *
 */
public class SmsChessServer {
	public static final Logger logger = Logger.getLogger(SmsChessServer.class);

	public static void main(String[] args) throws Exception {
		Configuration configuration = new Configuration();
		CmdLineParser parser = new CmdLineParser(configuration);
		try {
			parser.parseArgument(args);
		} catch (CmdLineException e) {
			System.out.println(e.getMessage());
			System.out.println("Available options:");
			parser.printUsage(System.out);
			return;
		}

		logger.info("Starting SmsChessServer...");

		final Properties props = new Properties();
		props.load(SmsChessServer.class.getClassLoader().getResourceAsStream(
				"connector.properties"));
		final String connectorClass = props.getProperty("connector");
		Connector connector = (Connector) Class.forName(connectorClass)
				.getConstructor(new Class[] { Configuration.class })
				.newInstance(new Object[] { configuration });

		XBoardEngineProcessManager engineProcessManager = new XBoardEngineProcessManager(
				configuration);
		final SessionManager sessionManager = new SessionManager(
				engineProcessManager, connector);
		MessageParser chessMessageParser = new MessageParserImpl(
				sessionManager);
		connector.openConnection(chessMessageParser);

		Runtime.getRuntime().addShutdownHook(new Thread() {
			@Override
			public void run() {
				for (XBoardSession session : sessionManager.getXBoardSessions()
						.values()) {
					session.terminate(false);
				}
			}
		});

		connector.runReceiver();
	}
}
