/**
 *
 */
package edu.agh.sm2006.smschess.server.cimd;

import org.apache.log4j.Logger;

import edu.agh.sm2006.smschess.server.Connector;
import edu.agh.sm2006.smschess.server.cimd.message.Message;
import edu.agh.sm2006.smschess.server.message.MalformedMessageException;
import edu.agh.sm2006.smschess.server.message.MessageParser;
import edu.agh.sm2006.smschess.server.message.client2server.ClientToServerMessage;

/**
 * @author Kuba Madej
 * 
 */
public class DeliverMessageConsumer implements CIMDMessageConsumer {
	private static final Logger logger = Logger
			.getLogger(DeliverMessageConsumer.class);

	private Connector connector;

	MessageParser parser;

	/**
	 * @param connector
	 * @param parser
	 */
	public DeliverMessageConsumer(Connector connector, MessageParser parser) {
		this.connector = connector;
		this.parser = parser;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.agh.sm2006.smschess.server.cimd.CIMDMessageConsumer#consume(edu.agh.sm2006.smschess.server.cimd.message.Message)
	 */
	public void consume(Message message) {
		String payload = null;
		String senderAddress = null;
		for (CIMDParameter parameter : message.getParameters()) {
			if (parameter.getType() == CIMDParameterType.UserData)
				payload = parameter.getValue();
			else if (parameter.getType() == CIMDParameterType.OriginatorAddress)
				senderAddress = parameter.getValue();
		}

		logger
				.debug("Received message '" + payload + "' from "
						+ senderAddress);

		final ClientToServerMessage clientToServerMessage;

		try {
			clientToServerMessage = parser.parse(payload);
		} catch (MalformedMessageException e) {
			logger.warn("Malformed/unsupported message received: '" + payload
					+ "'");
			return;
		}

		clientToServerMessage.setSenderId(senderAddress);

		final String response = clientToServerMessage.process();
		if (response != null) {
			logger.debug("Sending response message '" + response.toString()
					+ "' to " + senderAddress);
			connector.sendMessage(senderAddress, response);
		}

	}

}
