/**
 *
 */
package edu.agh.sm2006.smschess.server.cimd.message;

import java.util.HashMap;
import java.util.Map;

/**
 * Enum representing all possible CIMD message types.
 *
 * @author Kuba Madej
 *
 */
public enum MessageType {
	LoginRequest(1),
	LoginResponse(51),
	LogoutRequest(2),
	LogoutResponse(52),
	SubmitMessageRequest(3),
	SubmitMessageResponse(53),
	EnquireMessageStatusRequest(4),
	EnquireMessageStatusResponse(54),
	DeliveryRequest(5),
	DeliveryResponse(55),
	CancelMessageRequest(6),
	CancelMessageResponse(56),
	SetMessageRequest(8),
	SetMessageResponse(58),
	GetMessageRequest(9),
	GetMessageResponse(59),
	AliveRequest(40),
	AliveResponse(90),
	GeneralErrorResponse(98),
	Nack(99),
	DeliverMessage(20),
	DeliverMessageResponse(70),
	DeliverStatusReport(23),
	DeliverStatusReportResponse(73);

	private int messageCode;

	private static Map<String, MessageType> messageTypes;

	private static Map<String, MessageType> getMap() {
		if (messageTypes == null)
			messageTypes = new HashMap<String, MessageType>();
		return messageTypes;
	}

	private MessageType(int code) {
		messageCode = code;
		getMap().put(this.toString(), this);
	}

	public static MessageType getCIMDMessageType(final String code) {
		return messageTypes.get(code);
	}

	public static MessageType getResponseType(MessageType requestMessage) {
		return messageTypes.get(String.format("%02d",
				requestMessage.messageCode + 50));
	}

	public boolean isResponse() {
		return messageCode >= 50;
	}

	public String toString() {
		return String.format("%02d", messageCode);
	}
}
