/**
 *
 */
package edu.agh.sm2006.smschess.server.message;

import org.apache.commons.lang.ArrayUtils;
import org.apache.log4j.Logger;

import edu.agh.sm2006.smschess.server.message.client2server.AbstractClientToServerMessage;
import edu.agh.sm2006.smschess.server.session.SessionManager;

/**
 * 
 * Responsibility of this class is to parse messages sent to the server.
 * 
 * @author Kuba Madej
 * 
 */
public class MessageParserImpl implements MessageParser {
	private static final Logger logger = Logger
			.getLogger(MessageParserImpl.class);

	private SessionManager sessionManager;

	public MessageParserImpl(SessionManager sessionManager) {
		this.sessionManager = sessionManager;
	}

	/**
	 * @return parsed message or null if message is of unknown type
	 * @throws MalformedMessageException
	 */
	public AbstractClientToServerMessage parse(String payload)
			throws MalformedMessageException {
		AbstractClientToServerMessage message = null;
		try {
			final Class<? extends AbstractClientToServerMessage> messageClass = MessageType
					.getMessageClass(payload.charAt(0));
			if (messageClass == null)
				throw new MalformedMessageException();
			message = messageClass.newInstance();
		} catch (InstantiationException e) {
			logger.fatal("Fatal error: ", e);
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			logger.fatal("Fatal error: ", e);
			throw new RuntimeException(e);
		}
		String[] splitted = payload.split(":");
		if (payload.endsWith(":"))
			splitted = (String[]) ArrayUtils.add(splitted, "");
		message.setParameters((String[]) ArrayUtils.subarray(splitted, 1,
				splitted.length));
		message.setSessionManager(sessionManager);
		return message;
	}
}
