/**
 *
 */
package edu.agh.sm2006.smschess.server.message;

import java.util.HashMap;
import java.util.Map;

import edu.agh.sm2006.smschess.server.message.client2server.AbstractClientToServerMessage;
import edu.agh.sm2006.smschess.server.message.client2server.CreateSessionMessage;
import edu.agh.sm2006.smschess.server.message.client2server.JoinSessionMessage;
import edu.agh.sm2006.smschess.server.message.client2server.MoveMessage;
import edu.agh.sm2006.smschess.server.message.client2server.QuitMessage;

/**
 * Associates message types and their classes. Server to client may have null
 * classes as these are used only to parse client to server messages.
 *
 * @author Kuba Madej
 *
 */
public enum MessageType {
	Move('m', MoveMessage.class),
	CreateSession('c', CreateSessionMessage.class),
	JoinSession('j', JoinSessionMessage.class),
	Quit('q', QuitMessage.class),
	SessionJoined('x', null),
	Exception('e', null),
	Success('s', null),
	Go('g', null),
	GameOver('o', null);

	private char code;

	private static Map<Character, Class<? extends AbstractClientToServerMessage>> messages;

	private static Map<Character, Class<? extends AbstractClientToServerMessage>> getMap() {
		if (messages == null)
			messages = new HashMap<Character, Class<? extends AbstractClientToServerMessage>>();
		return messages;
	}

	public static Class<? extends AbstractClientToServerMessage> getMessageClass(
			char code) {
		return messages.get(code);
	}

	private MessageType(char code,
			Class<? extends AbstractClientToServerMessage> messageClass) {
		getMap().put(code, messageClass);
		this.code = code;
	}

	/**
	 * @return the code
	 */
	public String toString() {
		return "" + code;
	}
}
