/**
 *
 */
package edu.agh.sm2006.smschess.server.message.client2server;

import edu.agh.sm2006.smschess.server.message.MalformedMessageException;
import edu.agh.sm2006.smschess.server.message.server2client.IllegalSessionIdMessage;
import edu.agh.sm2006.smschess.server.message.server2client.SessionJoinedMessage;
import edu.agh.sm2006.smschess.server.message.server2client.SuccessMessage;
import edu.agh.sm2006.smschess.server.message.server2client.XBoardSessionNotCreatedMessage;
import edu.agh.sm2006.smschess.server.session.InvalidSessionIdException;
import edu.agh.sm2006.smschess.server.session.XBoardSessionsLimitExceededException;

/**
 * Message sent by client which wishes to create a new session. It's String
 * representation looks like: c:[sessionId]:[x/p] . Session id is passed only by
 * clients wishing to create new human to human session, otherwise it should be
 * left empty, and additional parameter of value 'x' should be passed. If client
 * wishes to create a new human to human session and wishes it to be public
 * session than the additional parameter should have a value of 'p', otherwise
 * the session created is private session.
 * 
 * @author Kuba Madej
 * 
 */
public class CreateSessionMessage extends AbstractClientToServerMessage {
	private static final String PUBLIC_SESSION = "p";

	private static final String PRIVATE_SESSION = "";

	private static final String XBOARD_SESSION = "x";

	boolean isXboardSession;

	boolean isPublicSession;

	public String process() {
		if (isXboardSession) {
			String sessionId;
			try {
				sessionId = sessionManager.createSession(senderId);
			} catch (XBoardSessionsLimitExceededException e) {
				return new XBoardSessionNotCreatedMessage().toString();
			}
			return new SessionJoinedMessage(sessionId).toString();
		}
		try {
			sessionManager.createSession(senderId, sessionId, isPublicSession);
		} catch (InvalidSessionIdException e) {
			return new IllegalSessionIdMessage().toString();
		}
		return new SuccessMessage().toString();
	}

	public void setParameters(String... parameters)
			throws MalformedMessageException {
		super.setParameters(parameters);

		if (parameters.length < 2)
			throw new MalformedMessageException();

		final String sessionType = parameters[1];

		if (!(PRIVATE_SESSION.equals(sessionType)
				|| PUBLIC_SESSION.equals(sessionType) || XBOARD_SESSION
				.equals(sessionType)))
			throw new MalformedMessageException();

		isXboardSession = XBOARD_SESSION.equals(sessionType);
		isPublicSession = PUBLIC_SESSION.equals(sessionType);
	}

}
