/**
 *
 */
package edu.agh.sm2006.smschess.server.session;

import ictk.boardgame.IllegalMoveException;
import ictk.boardgame.chess.Bishop;
import ictk.boardgame.chess.ChessBoard;
import ictk.boardgame.chess.ChessGame;
import ictk.boardgame.chess.ChessMove;
import ictk.boardgame.chess.ChessPiece;
import ictk.boardgame.chess.Knight;
import ictk.boardgame.chess.Queen;
import ictk.boardgame.chess.Rook;

import java.util.Date;

import edu.agh.sm2006.smschess.server.Connector;

/**
 * @author Kuba Madej
 *
 */
public abstract class Session {
	protected Date lastMove = new Date();

	protected ChessGame chessGame = new ChessGame();

	/**
	 * Package access to make testing possible
	 */
	ChessBoard chessBoard = (ChessBoard) chessGame.getBoard();

	protected String whitePlayer;

	protected Connector connector;

	protected SessionManager sessionManager;

	protected String sessionId;

	/**
	 *
	 * @param userId
	 *            user id (phone number) of user who opens the session
	 * @param xBoard
	 *            wheather this is going to be session against xboard engine
	 */
	public Session(final String sessionId, SessionManager sessionManager,
			Connector connector, String userId) {
		this.sessionId = sessionId;
		this.sessionManager = sessionManager;
		this.connector = connector;
		whitePlayer = userId;
	}

	public String getPrimaryPlayerId() {
		return whitePlayer;
	}

	/**
	 *
	 * @param move
	 * @return true if this session can be continued, false otherwise
	 * @throws IllegalMoveException
	 */
	public boolean move(String move) throws IllegalMoveException {
		chessBoard.playMove(parseMove(move));
		lastMove = new Date();
		return chessGame.getCurrentResult().isUndecided();
	}

	private ChessMove parseMove(final String moveString)
			throws IllegalMoveException {
		int originFile, originRow, destinationFile, destinationRow;
		originFile = moveString.charAt(0) - 'a' + 1;
		originRow = moveString.charAt(1) - '0';
		destinationFile = moveString.charAt(2) - 'a' + 1;
		destinationRow = moveString.charAt(3) - '0';

		// check for possible castle
		final ChessPiece piece = chessBoard.getSquare(originFile, originRow)
				.getOccupant();
		if (piece != null && piece.isKing()) {
			if (moveString.equals("e1g1") || moveString.equals("e8g8")) {
				return new ChessMove(chessBoard, ChessMove.CASTLE_KINGSIDE);
			} else if (moveString.equals("e1c1") || moveString.equals("e8c8")) {
				return new ChessMove(chessBoard, ChessMove.CASTLE_QUEENSIDE);
			}
		}

		// check for possible promotion
		if (moveString.length() > 4) {
			int promotion = Knight.INDEX;
			switch (moveString.charAt(4)) {
			case 'q':
				promotion = Queen.INDEX;
				break;
			case 'r':
				promotion = Rook.INDEX;
				break;
			case 'b':
				promotion = Bishop.INDEX;
			}
			return new ChessMove(chessBoard, originFile, originRow,
					destinationFile, destinationRow, promotion);
		}

		// normal move
		return new ChessMove(chessBoard, originFile, originRow,
				destinationFile, destinationRow);
	}

	/**
	 *
	 * @param notify
	 * @param notify
	 *            to notify or not other clients
	 */
	public abstract void terminate(boolean notify);

	/**
	 * @return the lastMove
	 */
	public Date getLastMove() {
		return lastMove;
	}

	/**
	 * @return the sessionId
	 */
	public String getSessionId() {
		return sessionId;
	}

}
