/**
 *
 */
package edu.agh.sm2006.smschess.server.session;

import ictk.boardgame.IllegalMoveException;

import org.apache.log4j.Logger;

import edu.agh.sm2006.smschess.server.Connector;
import edu.agh.sm2006.smschess.server.message.server2client.MoveMessage;
import edu.agh.sm2006.smschess.server.message.server2client.ServerFailureMessage;
import edu.agh.sm2006.smschess.server.session.xboard.XBoardEngineProcess;
import edu.agh.sm2006.smschess.server.session.xboard.XBoardEngineProcessException;
import edu.agh.sm2006.smschess.server.session.xboard.XBoardEngineProcessManager;

/**
 * XBoard session.
 *
 * @author Kuba Madej
 *
 */
public class XBoardSession extends Session {
	private static final Logger logger = Logger.getLogger(XBoardSession.class);

	private XBoardEngineProcessManager engineProcessManager;

	private XBoardEngineProcess engine;

	public XBoardSession(final String sessionId, SessionManager sessionManager,
			XBoardEngineProcessManager engineProcessManager,
			Connector connector, String userId)
			throws XBoardSessionsLimitExceededException {
		super(sessionId, sessionManager, connector, userId);
		this.engineProcessManager = engineProcessManager;
		try {
			engine = engineProcessManager.createProcess(sessionId, false);
		} catch (XBoardEngineProcessException e) {
			// shouldn't happen with stable engine!
			logger.fatal(e);
			connector.sendMessage(whitePlayer, new ServerFailureMessage()
					.toString());
		}
		if (engine == null) {
			throw new XBoardSessionsLimitExceededException();
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see edu.agh.sm2006.smschess.server.session.Session#move(java.lang.String)
	 */
	@Override
	public boolean move(String move) throws IllegalMoveException {
		final boolean toContinue = super.move(move);
		if (toContinue) {
			try {
				engine.move(move);
			} catch (XBoardEngineProcessException e) {
				// shouldn't happen with stable engine!
				logger.fatal(e);
				connector.sendMessage(whitePlayer, new ServerFailureMessage()
						.toString());
				return false;
			}
			new Thread(new Runnable() {

				public void run() {
					String engineMove;
					try {
						engineMove = engine.getMove();
					} catch (XBoardEngineProcessException e1) {
						// shouldn't happen with stable engine!
						logger.fatal(e1);
						connector.sendMessage(whitePlayer,
								new ServerFailureMessage().toString());
						sessionManager.terminateSession(sessionId);
						return;
					}
					boolean toContinue = false;
					try {
						toContinue = XBoardSession.super.move(engineMove);
					} catch (IllegalMoveException e) {
						// ignore, the assumption is that the engine makes only
						// valid moves - log just in case
						logger.fatal("Engine process makes illegal move!");
					}
					connector.sendMessage(whitePlayer, new MoveMessage(
							engineMove).toString());
					if (!toContinue)
						sessionManager.terminateSession(sessionId);
				}

			}).start();
		}
		return toContinue;
	}

	@Override
	public void terminate(boolean notify) {
		engine.destroy();
		engineProcessManager.releaseProcess();
	}
}
