package edu.agh.sm2006.smschess.server.session.xboard;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

/**
 *
 * @author Kuba Madej
 *
 */
public class XBoardEngineProcess {
	private static final String MOVE_COMMAND = "move ";

	private static final String PLAY_WHITE_COMMAND = "go\n";

	private static final String INITIAL_COMMAND = "xboard\nprotover 1\nnew\n";

	private static final String ENCODING = "US-ASCII";

	private Process process;

	private BufferedReader reader;

	private OutputStreamWriter writer;

	/**
	 * Constructs new process
	 *
	 * @param command
	 * @param playWhite
	 * @throws XBoardEngineProcessException
	 */
	public XBoardEngineProcess(final String command, final boolean playWhite)
			throws XBoardEngineProcessException {
		try {
			process = Runtime.getRuntime().exec(command);
			reader = new BufferedReader(new InputStreamReader(process
					.getInputStream(), ENCODING));
			writer = new OutputStreamWriter(process.getOutputStream(), ENCODING);
			writer.append(INITIAL_COMMAND);
			if (playWhite)
				writer.append(PLAY_WHITE_COMMAND);
			writer.flush();
		} catch (IOException e) {
			throw new XBoardEngineProcessException(e);
		}
	}

	/**
	 * Destroys this process
	 *
	 */
	public void destroy() {
		process.destroy();
	}

	/**
	 * Makes a move. The method doesn't care if the move is valid, it is
	 * caller's responsibility to ensure it.
	 *
	 * @param move
	 *            move to make
	 * @throws XBoardEngineProcessException
	 */
	public void move(String move) throws XBoardEngineProcessException {
		try {
			writer.append(move + "\n");
			writer.flush();
		} catch (IOException e) {
			throw new XBoardEngineProcessException(e);
		}
	}

	/**
	 *
	 * @return move made by the engine
	 * @throws XBoardEngineProcessException
	 */
	public String getMove() throws XBoardEngineProcessException {
		String command;
		do {
			try {
				command = reader.readLine();
			} catch (IOException e) {
				throw new XBoardEngineProcessException(e);
			}
		} while (!command.startsWith(MOVE_COMMAND));
		return command.substring(5);
	}

	public static void main(String[] args) throws XBoardEngineProcessException,
			IOException {
		XBoardEngineProcess process = new XBoardEngineProcess(
				"c:\\Documents and Settings\\sg0969438\\Desktop\\chess-engines\\aldebaran\\aldebaran.exe",
				false);
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				System.in));
		do {
			process.move(reader.readLine());
			System.out.println(process.getMove());
		} while (true);
	}
}
