/**
 *
 */
package edu.agh.sm2006.smschess.server.socket;

import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Scanner;

import edu.agh.sm2006.smschess.server.message.MalformedMessageException;
import edu.agh.sm2006.smschess.server.message.MessageParser;
import edu.agh.sm2006.smschess.server.message.client2server.ClientToServerMessage;

/**
 * @author Kuba Madej
 * 
 */
public class Client implements Runnable {
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(Client.class);

	private OutputStream out;

	private Scanner scanner;

	private MessageParser chessMessageParser;

	private int id;

	public void run() {
		while (scanner.hasNext()) {
			final String messageContent = scanner.next();
			ClientToServerMessage message;
			try {
				message = chessMessageParser.parse(messageContent);
			} catch (MalformedMessageException e) {
				logger.warn("Malformed/unsupported message received: '"
						+ messageContent + "'");
				continue;
			}

			logger.debug("Received '" + messageContent + "' from " + id);

			message.setSenderId(id + "");
			final String response = message.process();
			if (response != null) {
				sendMessage(response);
			}
		}
	}

	/**
	 * @param clientId
	 * @param socket
	 * @param chessMessageParser
	 */
	public Client(final int id, Socket socket, MessageParser chessMessageParser) {
		this.chessMessageParser = chessMessageParser;
		this.id = id;

		try {
			scanner = new Scanner(socket.getInputStream());
			scanner.useDelimiter("\n");

			out = socket.getOutputStream();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		new Thread(this).start();
	}

	public void sendMessage(final String message) {
		logger.debug("Sending '" + message + "' to " + id);
		try {
			out.write((message + "\n").getBytes());
			out.flush();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
